module hash.factory;

import hash.md2;
import hash.md4;
import hash.md5;
import hash.sha0;
import hash.sha1;
import hash.sha256;
import hash.sha512;
import hash.tiger;

private import std.string;

enum HASH { MD2, MD4, MD5, SHA0, SHA1, SHA256, SHA512, TIGER }
const char[][] types = ["MD2","MD4","MD5","SHA0","SHA1","SHA256","SHA512","TIGER"];

Hash produce(HASH type)
{
	if (type == HASH.MD2) return new Md2Hash();
	if (type == HASH.MD4) return new Md4Hash();
	if (type == HASH.MD5) return new Md5Hash();
	if (type == HASH.SHA0) return new Sha0Hash();
	if (type == HASH.SHA1) return new Sha1Hash();
	if (type == HASH.SHA256) return new Sha256Hash();
	if (type == HASH.SHA512) return new Sha512Hash();
	if (type == HASH.TIGER) return new TigerHash();
	throw new HashException("Unknown method");
}

Hash produce(char[] type)
{
	if (type !is null) {
		if (type.icmp("MD2") == 0) return new Md2Hash();
		if (type.icmp("MD4") == 0) return new Md4Hash();
		if (type.icmp("MD5") == 0) return new Md5Hash();
		if (type.icmp("SHA0") == 0) return new Sha0Hash();
		if (type.icmp("SHA1") == 0) return new Sha1Hash();
		if (type.icmp("SHA256") == 0) return new Sha256Hash();
		if (type.icmp("SHA512") == 0) return new Sha512Hash();
		if (type.icmp("Tiger") == 0) return new TigerHash();
	}
	throw new HashException("Unknown method");
}

unittest {
	foreach(char[] name; types) {
		assert(hash.factory.produce(name) !is null);
	}
	for(HASH i = HASH.MD2; i <= HASH.TIGER; i++) {
		assert(hash.factory.produce(i) !is null);
	}
}
